@@ -1,10 +1,12 @@
 <?php
 /**
  * @author Arthur Schiwon <blizzz@owncloud.com>
+ * @author Joas Schilling <nickvergessen@owncloud.com>
  * @author Jörn Friedrich Dreyer <jfd@butonic.de>
  * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
  *
- * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @copyright Copyright (c) 2016, ownCloud, Inc.
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -23,11 +25,17 @@
 
 namespace OCA\user_ldap\lib\user;
 
+use OC\Cache\CappedMemoryCache;
 use OCA\user_ldap\lib\user\IUserTools;
 use OCA\user_ldap\lib\user\User;
 use OCA\user_ldap\lib\LogWrapper;
 use OCA\user_ldap\lib\FilesystemHelper;
 use OCA\user_ldap\lib\user\OfflineUser;
+use OCP\IAvatarManager;
+use OCP\IConfig;
+use OCP\IDBConnection;
+use OCP\Image;
+use OCP\IUserManager;
 
 /**
  * Manager
@@ -39,10 +47,10 @@ class Manager {
 	/** @var IUserTools */
 	protected $access;
 
-	/** @var \OCP\IConfig */
+	/** @var IConfig */
 	protected $ocConfig;
 
-	/** @var \OCP\IDBConnection */
+	/** @var IDBConnection */
 	protected $db;
 
 	/** @var FilesystemHelper */
@@ -51,35 +59,35 @@ class Manager {
 	/** @var LogWrapper */
 	protected $ocLog;
 
-	/** @var \OCP\Image */
+	/** @var Image */
 	protected $image;
 
 	/** @param \OCP\IAvatarManager */
 	protected $avatarManager;
 
 	/**
-	 * array['byDN']	\OCA\user_ldap\lib\User[]
-	 * 	['byUid']	\OCA\user_ldap\lib\User[]
-	 * @var array $users
+	 * @var CappedMemoryCache $usersByDN
 	 */
-	protected $users = array(
-		'byDN'  => array(),
-		'byUid' => array(),
-	);
+	protected $usersByDN;
+	/**
+	 * @var CappedMemoryCache $usersByUid
+	 */
+	protected $usersByUid;
 
 	/**
-	 * @param \OCP\IConfig $ocConfig
+	 * @param IConfig $ocConfig
 	 * @param \OCA\user_ldap\lib\FilesystemHelper $ocFilesystem object that
 	 * gives access to necessary functions from the OC filesystem
 	 * @param  \OCA\user_ldap\lib\LogWrapper $ocLog
-	 * @param \OCP\IAvatarManager $avatarManager
-	 * @param \OCP\Image $image an empty image instance
-	 * @param \OCP\IDBConnection $db
-	 * @throws Exception when the methods mentioned above do not exist
+	 * @param IAvatarManager $avatarManager
+	 * @param Image $image an empty image instance
+	 * @param IDBConnection $db
+	 * @throws \Exception when the methods mentioned above do not exist
 	 */
-	public function __construct(\OCP\IConfig $ocConfig,
-		FilesystemHelper $ocFilesystem, LogWrapper $ocLog,
-		\OCP\IAvatarManager $avatarManager, \OCP\Image $image, \OCP\IDBConnection $db) {
+	public function __construct(IConfig $ocConfig,
+								FilesystemHelper $ocFilesystem, LogWrapper $ocLog,
+								IAvatarManager $avatarManager, Image $image,
+								IDBConnection $db, IUserManager $userManager) {
 
 		$this->ocConfig      = $ocConfig;
 		$this->ocFilesystem  = $ocFilesystem;
@@ -87,6 +95,9 @@ class Manager {
 		$this->avatarManager = $avatarManager;
 		$this->image         = $image;
 		$this->db            = $db;
+		$this->userManager   = $userManager;
+		$this->usersByDN     = new CappedMemoryCache();
+		$this->usersByUid    = new CappedMemoryCache();
 	}
 
 	/**
@@ -101,23 +112,23 @@ class Manager {
 	/**
 	 * @brief creates an instance of User and caches (just runtime) it in the
 	 * property array
-	 * @param string the DN of the user
-	 * @param string the internal (owncloud) username
-	 * @return \OCA\user_ldap\lib\User
+	 * @param string $dn the DN of the user
+	 * @param string $uid the internal (owncloud) username
+	 * @return \OCA\user_ldap\lib\User\User
 	 */
 	private function createAndCache($dn, $uid) {
 		$this->checkAccess();
 		$user = new User($uid, $dn, $this->access, $this->ocConfig,
 			$this->ocFilesystem, clone $this->image, $this->ocLog,
-			$this->avatarManager);
-		$this->users['byDN'][$dn]   = $user;
-		$this->users['byUid'][$uid] = $user;
+			$this->avatarManager, $this->userManager);
+		$this->usersByDN[$dn]   = $user;
+		$this->usersByUid[$uid] = $user;
 		return $user;
 	}
 
 	/**
 	 * @brief checks whether the Access instance has been set
-	 * @throws Exception if Access has not been set
+	 * @throws \Exception if Access has not been set
 	 * @return null
 	 */
 	private function checkAccess() {
@@ -190,7 +201,7 @@ class Manager {
 
 	/**
 	 * @brief returns a User object by it's ownCloud username
-	 * @param string the DN or username of the user
+	 * @param string $id the DN or username of the user
 	 * @return \OCA\user_ldap\lib\user\User|\OCA\user_ldap\lib\user\OfflineUser|null
 	 */
 	protected function createInstancyByUserName($id) {
@@ -207,16 +218,16 @@ class Manager {
 
 	/**
 	 * @brief returns a User object by it's DN or ownCloud username
-	 * @param string the DN or username of the user
+	 * @param string $id the DN or username of the user
 	 * @return \OCA\user_ldap\lib\user\User|\OCA\user_ldap\lib\user\OfflineUser|null
 	 * @throws \Exception when connection could not be established
 	 */
 	public function get($id) {
 		$this->checkAccess();
-		if(isset($this->users['byDN'][$id])) {
-			return $this->users['byDN'][$id];
-		} else if(isset($this->users['byUid'][$id])) {
-			return $this->users['byUid'][$id];
+		if(isset($this->usersByDN[$id])) {
+			return $this->usersByDN[$id];
+		} else if(isset($this->usersByUid[$id])) {
+			return $this->usersByUid[$id];
 		}
 
 		if($this->access->stringResemblesDN($id) ) {
